package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Testbed;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.SfaConnectionPool;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import be.iminds.ilabt.jfed.util.RFC3339Util;
import be.iminds.ilabt.jfed.util.TimeUtil;
import com.sun.javafx.webkit.WebConsoleListener;
import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.stage.Window;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.net.URL;
import java.text.ParseException;
import java.time.Duration;
import java.time.Instant;

public class TestbedSiteDialog extends JFedBrowser {
    private static final Logger LOG = LoggerFactory.getLogger(TestbedSiteDialog.class);

    private Testbed testbed;
    private String url;
    private String title;

    public static class TestbedSiteDialogJavaScriptBridge extends JavaScriptBridge {
        @Nonnull private final TestbedSiteDialog testbedSiteDialog;
        public TestbedSiteDialogJavaScriptBridge(@Nonnull TestbedSiteDialog testbedSiteDialog) {
            super(testbedSiteDialog);
            this.testbedSiteDialog = testbedSiteDialog;
        }

        public void addRequestRspec(String requestRSpec) {
            //todo
        }

        public void setRequestRspec(String requestRSpec) {
            //todo
        }

        public void addNode(String urn) {
            //todo
        }

        public void close() {
            LOG.debug("TestbedSiteDialogJavaScriptBridge.close");
            Window stage = testbedSiteDialog.scene.getWindow();
            stage.hide();
        }
    }

    @Inject
    public TestbedSiteDialog(@Nonnull GeniUserProvider geniUserProvider,
                             @Nonnull SfaConnectionPool connectionPool,
                             @Nonnull JFedTrustStore trustStore,
                             @Nonnull JFedPreferences preferences) {
        super(geniUserProvider, connectionPool, trustStore, preferences);
    }

    public void setUrlAndTestbed(@Nonnull URL url,
                                 @Nonnull Testbed testbed,
                                 @Nonnull String title) {
        this.url = url.toExternalForm();
        this.testbed = testbed;
        this.title = title;

        loadPageWhenReady();
    }

    @FXML
    @Override
    protected void initialize() {
        webEngine = webView.getEngine();

        URLStreamFactoryCustomizer.useDedicatedProxyForWebkit(
                new MyHttpUrlStreamHandler(), "https");
        initialized = true;

        webEngine.getLoadWorker().stateProperty().addListener(
                (ObservableValue<? extends Worker.State> ov, Worker.State oldState,
                 Worker.State newState) -> {
                    if (newState == Worker.State.SUCCEEDED) {
                        jsBridgeBackoff = 1;
                        registerJsBridge();
                    }
                });

        WebConsoleListener.setDefaultListener((webView, message, lineNumber, sourceId) -> {
            LOG.debug("[Webengine message] "+message + "[at " + lineNumber + "]");
        });

        loadPageWhenReady();
    }

    @Override
    protected void loadPageWhenReady() {
        if (!initialized)
            return;

        if (this.testbed == null)
            return;

        Platform.runLater(this::registerJsBridge);
        loadPage();
    }

    @Override
    protected Object makeJavascriptBridge() {
        return new TestbedSiteDialogJavaScriptBridge(this);
    }

    protected void loadPage() {
        headerLabel.setText(title);
        webEngine.load(url);
    }

}
